@@ -18,14 +18,18 @@
 # under the License.
 
 """
-Host-side wrapper: workflow provides only --arch.
-Images are defined as regular Python lists (no env vars).
+Host-side wrapper for building Python wheels in manylinux containers.
+
+Usage:
+  ./build_linux_wheels.py --arch X86 --python cp38-cp38
+  ./build_linux_wheels.py --arch AARCH64 --python cp313-cp313 --release
 
 Environment:
   - GITHUB_WORKSPACE (optional; defaults to cwd)
 """
 
 from __future__ import annotations
+
 import argparse
 import os
 import shlex
@@ -33,27 +37,15 @@ import subprocess
 import sys
 from typing import List
 
-# Define Python version sets directly in the Python script
-RELEASE_PYTHON_VERSIONS = (
-    "cp38-cp38 cp39-cp39 cp310-cp310 cp311-cp311 cp312-cp312 cp313-cp313"
-)
-DEFAULT_PYTHON_VERSIONS = "cp38-cp38 cp313-cp313"
-
 # Path to the container build script
 CONTAINER_SCRIPT_PATH = "ci/tasks/python_container_build_script.sh"
 
 DEFAULT_X86_IMAGES = [
     "quay.io/pypa/manylinux2014_x86_64:latest",
-    # "quay.io/pypa/manylinux_2_28_x86_64:latest",
-    # bazel binaries do not work with musl
-    # "quay.io/pypa/musllinux_1_2_x86_64:latest",
 ]
 
 DEFAULT_AARCH64_IMAGES = [
     "quay.io/pypa/manylinux2014_aarch64:latest",
-    # "quay.io/pypa/manylinux_2_28_aarch64:latest",
-    # bazel binaries do not work with musl
-    # "quay.io/pypa/musllinux_1_2_aarch64:latest",
 ]
 
 ARCH_ALIASES = {
@@ -73,10 +65,11 @@ def parse_args():
         "--arch", required=True, help="Architecture (e.g. X86, X64, AARCH64)"
     )
     p.add_argument(
-        "--release", action="store_true", help="Run full test suite for release"
+        "--python", required=True, help="Python version (e.g. cp38-cp38, cp313-cp313)"
     )
+    p.add_argument("--release", action="store_true", help="Run in release mode")
     p.add_argument(
-        "--dry-run", action="store_true", help="Print docker commands without running"
+        "--dry-run", action="store_true", help="Print docker command without running"
     )
     return p.parse_args()
 
@@ -86,21 +79,19 @@ def normalize_arch(raw: str) -> str:
     return ARCH_ALIASES.get(key, raw.strip().lower())
 
 
-def collect_images_for_arch(arch_normalized: str) -> List[str]:
+def get_image_for_arch(arch_normalized: str) -> str:
     if arch_normalized == "x86":
-        imgs = DEFAULT_X86_IMAGES  # dedupe preserving order
+        return DEFAULT_X86_IMAGES[0]
     elif arch_normalized == "arm64":
-        imgs = DEFAULT_AARCH64_IMAGES
+        return DEFAULT_AARCH64_IMAGES[0]
     else:
         raise SystemExit(f"Unsupported arch: {arch_normalized!r}")
-    return imgs
 
 
-def build_docker_cmd(workspace: str, image: str, release: bool = False) -> List[str]:
+def build_docker_cmd(
+    workspace: str, image: str, python_version: str, release: bool = False
+) -> List[str]:
     workspace = os.path.abspath(workspace)
-    python_versions = RELEASE_PYTHON_VERSIONS if release else DEFAULT_PYTHON_VERSIONS
-
-    # Get GitHub reference name from environment
     github_ref_name = os.environ.get("GITHUB_REF_NAME", "")
 
     cmd = [
@@ -109,16 +100,15 @@ def build_docker_cmd(workspace: str, image: str, release: bool = False) -> List[
         "-i",
         "--rm",
         "-v",
-        f"{workspace}:/work",  # (v)olume
+        f"{workspace}:/work",
         "-w",
-        "/work",  # (w)orking directory
+        "/work",
         "-e",
-        f"PYTHON_VERSIONS={python_versions}",  # (e)nvironment variables
+        f"PYTHON_VERSIONS={python_version}",
         "-e",
         f"RELEASE_BUILD={'1' if release else '0'}",
     ]
 
-    # Pass GitHub reference name if available
     if github_ref_name:
         cmd.extend(["-e", f"GITHUB_REF_NAME={github_ref_name}"])
 
@@ -126,52 +116,32 @@ def build_docker_cmd(workspace: str, image: str, release: bool = False) -> List[
     return cmd
 
 
-def run_for_images(
-    images: List[str], workspace: str, dry_run: bool, release: bool = False
-) -> int:
-    rc_overall = 0
-    for image in images:
-        docker_cmd = build_docker_cmd(workspace, image, release=release)
-        printable = " ".join(shlex.quote(c) for c in docker_cmd)
-        print(f"+ {printable}")
-        if dry_run:
-            continue
-        try:
-            completed = subprocess.run(docker_cmd)
-            if completed.returncode != 0:
-                print(
-                    f"Container {image} exited with {completed.returncode}",
-                    file=sys.stderr,
-                )
-                rc_overall = completed.returncode if rc_overall == 0 else rc_overall
-            else:
-                print(f"Container {image} completed successfully.")
-        except KeyboardInterrupt:
-            print("Interrupted by user", file=sys.stderr)
-            return 130
-        except FileNotFoundError as e:
-            print(f"Error running docker: {e}", file=sys.stderr)
-            return 2
-    return rc_overall
-
-
 def main() -> int:
     args = parse_args()
     arch = normalize_arch(args.arch)
-    images = collect_images_for_arch(arch)
-    if not images:
-        print(f"No images configured for arch {arch}", file=sys.stderr)
-        return 2
+    image = get_image_for_arch(arch)
     workspace = os.environ.get("GITHUB_WORKSPACE", os.getcwd())
 
-    # Check if the container script exists
     script_path = os.path.join(workspace, CONTAINER_SCRIPT_PATH)
     if not os.path.exists(script_path):
         print(f"Container script not found at {script_path}", file=sys.stderr)
         return 2
 
-    print(f"Selected images for arch {args.arch}: {images}")
-    return run_for_images(images, workspace, args.dry_run, release=args.release)
+    docker_cmd = build_docker_cmd(workspace, image, args.python, release=args.release)
+    printable = " ".join(shlex.quote(c) for c in docker_cmd)
+    print(f"+ {printable}")
+
+    if args.dry_run:
+        return 0
+
+    try:
+        completed = subprocess.run(docker_cmd)
+        if completed.returncode != 0:
+            print(f"Container exited with {completed.returncode}", file=sys.stderr)
+        return completed.returncode
+    except FileNotFoundError as e:
+        print(f"Error running docker: {e}", file=sys.stderr)
+        return 2
 
 
 if __name__ == "__main__":
