@@ -1207,6 +1207,9 @@ public class ClassResolver extends TypeResolver {
           }
           return false;
         };
+    if (classChecker instanceof AllowListChecker) {
+      ((AllowListChecker) classChecker).addListener(this);
+    }
   }
 
   @Override
@@ -1326,7 +1329,8 @@ public class ClassResolver extends TypeResolver {
     }
   }
 
-  private Serializer createSerializer(Class<?> cls) {
+  @Internal
+  public Serializer createSerializer(Class<?> cls) {
     DisallowedList.checkNotInDisallowedList(cls.getName());
     if (!isSecure(cls)) {
       throw new InsecureException(generateSecurityMsg(cls));
@@ -1335,7 +1339,8 @@ public class ClassResolver extends TypeResolver {
           && !Functions.isLambda(cls)
           && !ReflectionUtils.isJdkProxy(cls)
           && !extRegistry.registeredClassIdMap.containsKey(cls)
-          && !shimDispatcher.contains(cls)) {
+          && !shimDispatcher.contains(cls)
+          && !extRegistry.isTypeCheckerSet()) {
         LOG.warn(generateSecurityMsg(cls));
       }
     }
