@@ -42,9 +42,12 @@ import java.util.Set;
 import java.util.SortedSet;
 import java.util.TreeSet;
 import java.util.Vector;
+import java.util.concurrent.ArrayBlockingQueue;
 import java.util.concurrent.ConcurrentHashMap;
 import java.util.concurrent.ConcurrentSkipListSet;
 import java.util.concurrent.CopyOnWriteArrayList;
+import java.util.concurrent.CopyOnWriteArraySet;
+import java.util.concurrent.LinkedBlockingQueue;
 import org.apache.fory.Fory;
 import org.apache.fory.collection.CollectionSnapshot;
 import org.apache.fory.exception.ForyException;
@@ -290,6 +293,51 @@ public class CollectionSerializers {
       Object[] elements = ((CollectionContainer) collection).elements;
       return new CopyOnWriteArrayList(elements);
     }
+
+    @Override
+    public CopyOnWriteArrayList copy(CopyOnWriteArrayList originCollection) {
+      CopyOnWriteArrayList newCollection = new CopyOnWriteArrayList();
+      if (needToCopyRef) {
+        fory.reference(originCollection, newCollection);
+      }
+      List copyList = new ArrayList(originCollection.size());
+      copyElements(originCollection, copyList);
+      newCollection.addAll(copyList);
+      return newCollection;
+    }
+  }
+
+  public static class CopyOnWriteArraySetSerializer
+      extends ConcurrentCollectionSerializer<CopyOnWriteArraySet> {
+
+    public CopyOnWriteArraySetSerializer(Fory fory, Class<CopyOnWriteArraySet> type) {
+      super(fory, type, true);
+    }
+
+    @Override
+    public Collection newCollection(MemoryBuffer buffer) {
+      int numElements = buffer.readVarUint32Small7();
+      setNumElements(numElements);
+      return new CollectionContainer<>(numElements);
+    }
+
+    @Override
+    public CopyOnWriteArraySet onCollectionRead(Collection collection) {
+      Object[] elements = ((CollectionContainer) collection).elements;
+      return new CopyOnWriteArraySet(Arrays.asList(elements));
+    }
+
+    @Override
+    public CopyOnWriteArraySet copy(CopyOnWriteArraySet originCollection) {
+      CopyOnWriteArraySet newCollection = new CopyOnWriteArraySet();
+      if (needToCopyRef) {
+        fory.reference(originCollection, newCollection);
+      }
+      List copyList = new ArrayList(originCollection.size());
+      copyElements(originCollection, copyList);
+      newCollection.addAll(copyList);
+      return newCollection;
+    }
   }
 
   public static final class EmptySetSerializer extends CollectionSerializer<Set<?>> {
@@ -690,6 +738,122 @@ public class CollectionSerializers {
     }
   }
 
+  /**
+   * Serializer for {@link ArrayBlockingQueue}.
+   *
+   * <p>This serializer handles the capacity field which is essential for ArrayBlockingQueue since
+   * it's a bounded queue. The capacity is stored in the items array length and needs to be
+   * preserved during serialization.
+   */
+  public static class ArrayBlockingQueueSerializer
+      extends ConcurrentCollectionSerializer<ArrayBlockingQueue> {
+
+    // Use reflection to get the items array length which represents the capacity.
+    // This avoids race conditions when reading remainingCapacity() and size() separately.
+    private static final long ITEMS_OFFSET;
+
+    static {
+      try {
+        Field itemsField = ArrayBlockingQueue.class.getDeclaredField("items");
+        ITEMS_OFFSET = Platform.objectFieldOffset(itemsField);
+      } catch (NoSuchFieldException e) {
+        throw new RuntimeException(e);
+      }
+    }
+
+    public ArrayBlockingQueueSerializer(Fory fory, Class<ArrayBlockingQueue> cls) {
+      super(fory, cls, true);
+    }
+
+    private static int getCapacity(ArrayBlockingQueue queue) {
+      Object[] items = (Object[]) Platform.getObject(queue, ITEMS_OFFSET);
+      return items.length;
+    }
+
+    @Override
+    public CollectionSnapshot onCollectionWrite(MemoryBuffer buffer, ArrayBlockingQueue value) {
+      // Read capacity before creating snapshot to ensure consistency
+      int capacity = getCapacity(value);
+      CollectionSnapshot snapshot = super.onCollectionWrite(buffer, value);
+      buffer.writeVarUint32Small7(capacity);
+      return snapshot;
+    }
+
+    @Override
+    public ArrayBlockingQueue newCollection(MemoryBuffer buffer) {
+      int numElements = buffer.readVarUint32Small7();
+      setNumElements(numElements);
+      int capacity = buffer.readVarUint32Small7();
+      ArrayBlockingQueue queue = new ArrayBlockingQueue<>(capacity);
+      fory.getRefResolver().reference(queue);
+      return queue;
+    }
+
+    @Override
+    public Collection newCollection(Collection collection) {
+      ArrayBlockingQueue abq = (ArrayBlockingQueue) collection;
+      int capacity = getCapacity(abq);
+      return new ArrayBlockingQueue<>(capacity);
+    }
+  }
+
+  /**
+   * Serializer for {@link LinkedBlockingQueue}.
+   *
+   * <p>This serializer handles the capacity field which is essential for LinkedBlockingQueue since
+   * it can be a bounded queue. The capacity needs to be preserved during serialization.
+   */
+  public static class LinkedBlockingQueueSerializer
+      extends ConcurrentCollectionSerializer<LinkedBlockingQueue> {
+
+    // Use reflection to get the capacity field directly.
+    // This avoids race conditions when reading remainingCapacity() and size() separately.
+    private static final long CAPACITY_OFFSET;
+
+    static {
+      try {
+        Field capacityField = LinkedBlockingQueue.class.getDeclaredField("capacity");
+        CAPACITY_OFFSET = Platform.objectFieldOffset(capacityField);
+      } catch (NoSuchFieldException e) {
+        throw new RuntimeException(e);
+      }
+    }
+
+    public LinkedBlockingQueueSerializer(Fory fory, Class<LinkedBlockingQueue> cls) {
+      super(fory, cls, true);
+    }
+
+    private static int getCapacity(LinkedBlockingQueue queue) {
+      return Platform.getInt(queue, CAPACITY_OFFSET);
+    }
+
+    @Override
+    public CollectionSnapshot onCollectionWrite(MemoryBuffer buffer, LinkedBlockingQueue value) {
+      // Read capacity before creating snapshot to ensure consistency
+      int capacity = getCapacity(value);
+      CollectionSnapshot snapshot = super.onCollectionWrite(buffer, value);
+      buffer.writeVarUint32Small7(capacity);
+      return snapshot;
+    }
+
+    @Override
+    public LinkedBlockingQueue newCollection(MemoryBuffer buffer) {
+      int numElements = buffer.readVarUint32Small7();
+      setNumElements(numElements);
+      int capacity = buffer.readVarUint32Small7();
+      LinkedBlockingQueue queue = new LinkedBlockingQueue<>(capacity);
+      fory.getRefResolver().reference(queue);
+      return queue;
+    }
+
+    @Override
+    public Collection newCollection(Collection collection) {
+      LinkedBlockingQueue lbq = (LinkedBlockingQueue) collection;
+      int capacity = getCapacity(lbq);
+      return new LinkedBlockingQueue<>(capacity);
+    }
+  }
+
   /**
    * Java serializer to serialize all fields of a collection implementation. Note that this
    * serializer won't use element generics and doesn't support JIT, performance won't be the best,
@@ -838,48 +1002,59 @@ public class CollectionSerializers {
 
   public static void registerDefaultSerializers(Fory fory) {
     ClassResolver resolver = fory.getClassResolver();
-    resolver.registerSerializer(ArrayList.class, new ArrayListSerializer(fory));
+    resolver.registerInternalSerializer(ArrayList.class, new ArrayListSerializer(fory));
     Class arrayAsListClass = Arrays.asList(1, 2).getClass();
-    resolver.registerSerializer(
+    resolver.registerInternalSerializer(
         arrayAsListClass, new ArraysAsListSerializer(fory, arrayAsListClass));
-    resolver.registerSerializer(
+    resolver.registerInternalSerializer(
         LinkedList.class, new CollectionSerializer(fory, LinkedList.class, true));
-    resolver.registerSerializer(HashSet.class, new HashSetSerializer(fory));
-    resolver.registerSerializer(LinkedHashSet.class, new LinkedHashSetSerializer(fory));
-    resolver.registerSerializer(TreeSet.class, new SortedSetSerializer<>(fory, TreeSet.class));
-    resolver.registerSerializer(
+    resolver.registerInternalSerializer(HashSet.class, new HashSetSerializer(fory));
+    resolver.registerInternalSerializer(LinkedHashSet.class, new LinkedHashSetSerializer(fory));
+    resolver.registerInternalSerializer(
+        TreeSet.class, new SortedSetSerializer<>(fory, TreeSet.class));
+    resolver.registerInternalSerializer(
         Collections.EMPTY_LIST.getClass(),
         new EmptyListSerializer(fory, (Class<List<?>>) Collections.EMPTY_LIST.getClass()));
-    resolver.registerSerializer(
+    resolver.registerInternalSerializer(
         Collections.emptySortedSet().getClass(),
         new EmptySortedSetSerializer(
             fory, (Class<SortedSet<?>>) Collections.emptySortedSet().getClass()));
-    resolver.registerSerializer(
+    resolver.registerInternalSerializer(
         Collections.EMPTY_SET.getClass(),
         new EmptySetSerializer(fory, (Class<Set<?>>) Collections.EMPTY_SET.getClass()));
-    resolver.registerSerializer(
+    resolver.registerInternalSerializer(
         Collections.singletonList(null).getClass(),
         new CollectionsSingletonListSerializer(
             fory, (Class<List<?>>) Collections.singletonList(null).getClass()));
-    resolver.registerSerializer(
+    resolver.registerInternalSerializer(
         Collections.singleton(null).getClass(),
         new CollectionsSingletonSetSerializer(
             fory, (Class<Set<?>>) Collections.singleton(null).getClass()));
-    resolver.registerSerializer(
+    resolver.registerInternalSerializer(
         ConcurrentSkipListSet.class,
         new ConcurrentSkipListSetSerializer(fory, ConcurrentSkipListSet.class));
-    resolver.registerSerializer(Vector.class, new VectorSerializer(fory, Vector.class));
-    resolver.registerSerializer(ArrayDeque.class, new ArrayDequeSerializer(fory, ArrayDeque.class));
-    resolver.registerSerializer(BitSet.class, new BitSetSerializer(fory, BitSet.class));
-    resolver.registerSerializer(
+    resolver.registerInternalSerializer(Vector.class, new VectorSerializer(fory, Vector.class));
+    resolver.registerInternalSerializer(
+        ArrayDeque.class, new ArrayDequeSerializer(fory, ArrayDeque.class));
+    resolver.registerInternalSerializer(BitSet.class, new BitSetSerializer(fory, BitSet.class));
+    resolver.registerInternalSerializer(
         PriorityQueue.class, new PriorityQueueSerializer(fory, PriorityQueue.class));
-    resolver.registerSerializer(
+    resolver.registerInternalSerializer(
+        ArrayBlockingQueue.class, new ArrayBlockingQueueSerializer(fory, ArrayBlockingQueue.class));
+    resolver.registerInternalSerializer(
+        LinkedBlockingQueue.class,
+        new LinkedBlockingQueueSerializer(fory, LinkedBlockingQueue.class));
+    resolver.registerInternalSerializer(
         CopyOnWriteArrayList.class,
         new CopyOnWriteArrayListSerializer(fory, CopyOnWriteArrayList.class));
     final Class setFromMapClass = Collections.newSetFromMap(new HashMap<>()).getClass();
-    resolver.registerSerializer(setFromMapClass, new SetFromMapSerializer(fory, setFromMapClass));
-    resolver.registerSerializer(
+    resolver.registerInternalSerializer(
+        setFromMapClass, new SetFromMapSerializer(fory, setFromMapClass));
+    resolver.registerInternalSerializer(
         ConcurrentHashMap.KeySetView.class,
         new ConcurrentHashMapKeySetViewSerializer(fory, ConcurrentHashMap.KeySetView.class));
+    resolver.registerInternalSerializer(
+        CopyOnWriteArraySet.class,
+        new CopyOnWriteArraySetSerializer(fory, CopyOnWriteArraySet.class));
   }
 }
