@@ -218,6 +218,13 @@ public class MemoryBufferTest {
     Assert.assertFalse(buf1.equalTo(buf2, 0, 0, buf1.size()));
   }
 
+  @Test
+  public void testEqualToZeroSize() {
+    MemoryBuffer buf1 = MemoryUtils.buffer(0);
+    MemoryBuffer buf2 = MemoryUtils.buffer(0);
+    Assert.assertTrue(buf1.equalTo(buf2, 0, 0, buf1.size()));
+  }
+
   @Test
   public void testWritePrimitiveArrayWithSizeEmbedded() {
     MemoryBuffer buf = MemoryUtils.buffer(16);
