@@ -97,7 +97,7 @@ public class MapSerializersTest extends ForyTestBase {
     byte[] bytes2 = fory.serialize(data);
     Assert.assertTrue(bytes1.length > bytes2.length);
     fory.getGenerics().popGenericType();
-    Assert.assertThrows(RuntimeException.class, () -> fory.deserialize(bytes2));
+    assertThrowsCause(RuntimeException.class, () -> fory.deserialize(bytes2));
 
     // testSortedMap
     Map<String, Integer> treeMap = new TreeMap<>(ImmutableMap.of("a", 1, "b", 2));
@@ -107,7 +107,7 @@ public class MapSerializersTest extends ForyTestBase {
     byte[] sortMapBytes2 = fory.serialize(treeMap);
     Assert.assertTrue(sortMapBytes1.length > sortMapBytes2.length);
     fory.getGenerics().popGenericType();
-    Assert.assertThrows(RuntimeException.class, () -> fory.deserialize(sortMapBytes2));
+    assertThrowsCause(RuntimeException.class, () -> fory.deserialize(sortMapBytes2));
 
     // testTreeMap
     TreeMap<String, String> map =
@@ -260,7 +260,7 @@ public class MapSerializersTest extends ForyTestBase {
     byte[] bytes2 = fory.serialize(data);
     Assert.assertTrue(bytes1.length > bytes2.length);
     fory.getGenerics().popGenericType();
-    Assert.assertThrows(RuntimeException.class, () -> fory.deserialize(bytes2));
+    assertThrowsCause(RuntimeException.class, () -> fory.deserialize(bytes2));
   }
 
   @Test(dataProvider = "referenceTrackingConfig")
@@ -278,7 +278,7 @@ public class MapSerializersTest extends ForyTestBase {
     byte[] bytes2 = fory.serialize(data);
     Assert.assertTrue(bytes1.length > bytes2.length);
     fory.getGenerics().popGenericType();
-    Assert.assertThrows(RuntimeException.class, () -> fory.deserialize(bytes2));
+    assertThrowsCause(RuntimeException.class, () -> fory.deserialize(bytes2));
   }
 
   @Test(dataProvider = "foryCopyConfig")
@@ -1048,4 +1048,30 @@ public class MapSerializersTest extends ForyTestBase {
     State state2 = (State) fory2.deserialize(bytes);
     Assert.assertEquals(state2.map.get("bar"), new String[] {"bar"});
   }
+
+  @Data
+  public static class OuterClass {
+    private Map<String, InnerClass> f1 = new HashMap<>();
+    private TestEnum f2;
+  }
+
+  @Data
+  public static class InnerClass {
+    int f1;
+  }
+
+  @Test(dataProvider = "compatible")
+  public void testNestedMapGenericCodegen(boolean compatible) {
+    Fory fory =
+        builder()
+            .withCodegen(true)
+            .withCompatibleMode(
+                compatible ? CompatibleMode.COMPATIBLE : CompatibleMode.SCHEMA_CONSISTENT)
+            .requireClassRegistration(false)
+            .build();
+
+    OuterClass value = new OuterClass();
+    value.f1.put("aaa", null);
+    serDeCheck(fory, value);
+  }
 }
