@@ -19,8 +19,8 @@
 
 package org.apache.fory.format.encoder;
 
-import org.apache.arrow.vector.types.pojo.Schema;
 import org.apache.fory.format.row.binary.BinaryRow;
+import org.apache.fory.format.type.Schema;
 
 /**
  * Encoder to encode/decode object to/from row. A RowEncoder instance is reusable but not
