@@ -19,6 +19,17 @@ import pyfory as fory
 
 from pyfory.tests.core import require_pyarrow
 from pyfory.util import lazy_import
+from pyfory.format import (
+    schema,
+    field,
+    int8,
+    int16,
+    int32,
+    int64,
+    utf8,
+    map_,
+    to_arrow_schema,
+)
 
 pa = lazy_import("pyarrow")
 
@@ -27,18 +38,23 @@ pa = lazy_import("pyarrow")
 def test_vectorized():
     field_names = ["f" + str(i) for i in range(1, 6)]
     cls = fory.record_class_factory("TEST_VECTORIZED", field_names)
-    schema = pa.schema(
+    # Create Fory schema for the encoder
+    fory_schema = schema(
         [
-            ("f1", pa.int64()),
-            ("f2", pa.int32()),
-            ("f3", pa.int16()),
-            ("f4", pa.int8()),
-            ("f5", pa.string()),
-        ],
-        metadata={"cls": fory.get_qualified_classname(cls)},
+            field("f1", int64()),
+            field("f2", int32()),
+            field("f3", int16()),
+            field("f4", int8()),
+            field("f5", utf8()),
+        ]
     )
-    writer = fory.format.ArrowWriter(schema)
-    encoder = fory.create_row_encoder(schema)
+    # Convert to Arrow schema for ArrowWriter
+    arrow_schema = to_arrow_schema(fory_schema)
+    # Add metadata for class resolution
+    arrow_schema = arrow_schema.with_metadata({"cls": fory.get_qualified_classname(cls)})
+
+    writer = fory.format.ArrowWriter(arrow_schema)
+    encoder = fory.create_row_encoder(fory_schema)
     num_rows = 10
     data = [[] for _ in range(len(field_names))]
     for i in range(num_rows):
@@ -56,7 +72,7 @@ def test_vectorized():
     assert record_batch.num_rows == num_rows
     assert record_batch.num_columns == 5
 
-    data = [pa.array(data[i], type=schema[i].type) for i in range(len(field_names))]
+    data = [pa.array(data[i], type=arrow_schema[i].type) for i in range(len(field_names))]
     batch1 = pa.RecordBatch.from_arrays(data, field_names)
     assert batch1 == record_batch
 
@@ -68,24 +84,31 @@ def test_vectorized():
 @require_pyarrow
 def test_vectorized_map():
     cls = fory.record_class_factory("TEST_VECTORIZED_MAP", ["f0"])
-    schema = pa.schema(
-        [("f0", pa.map_(pa.string(), pa.int32()))],
-        metadata={"cls": fory.get_qualified_classname(cls)},
+    # Create Fory schema for the encoder
+    fory_schema = schema(
+        [
+            field("f0", map_(utf8(), int32())),
+        ]
     )
-    print(schema)
-    writer = fory.format.ArrowWriter(schema)
-    encoder = fory.create_row_encoder(schema)
+    # Convert to Arrow schema for ArrowWriter
+    arrow_schema = to_arrow_schema(fory_schema)
+    # Add metadata for class resolution
+    arrow_schema = arrow_schema.with_metadata({"cls": fory.get_qualified_classname(cls)})
+    print(arrow_schema)
+
+    writer = fory.format.ArrowWriter(arrow_schema)
+    encoder = fory.create_row_encoder(fory_schema)
     num_rows = 5
     data = []
     for i in range(num_rows):
-        map_ = {"k1": 1, "k2": 2}
-        data.append(list(map_.items()))
-        obj = cls(f0=map_)
+        map_data = {"k1": 1, "k2": 2}
+        data.append(list(map_data.items()))
+        obj = cls(f0=map_data)
         row = encoder.to_row(obj)
         writer.write(row)
     record_batch = writer.finish()
     print(f"record_batch {record_batch}")
-    data = [pa.array(data, type=schema[0].type)]
+    data = [pa.array(data, type=arrow_schema[0].type)]
     batch1 = pa.RecordBatch.from_arrays(data, ["f0"])
     assert batch1 == record_batch
 
