@@ -56,8 +56,9 @@ pub fn deserialize_any_box(context: &mut ReadContext) -> Result<Box<dyn Any>, Er
     let typeinfo = context.read_any_typeinfo()?;
     // Check for generic container types which cannot be deserialized polymorphically
     check_generic_container_type(&typeinfo)?;
-    let deserializer_fn = typeinfo.get_harness().get_read_data_fn();
-    let result = deserializer_fn(context);
+    let result = typeinfo
+        .get_harness()
+        .read_polymorphic_data(context, &typeinfo);
     context.dec_depth();
     result
 }
@@ -217,8 +218,9 @@ pub fn read_box_any(
     };
     // Check for generic container types which cannot be deserialized polymorphically
     check_generic_container_type(&typeinfo)?;
-    let deserializer_fn = typeinfo.get_harness().get_read_data_fn();
-    let result = deserializer_fn(context);
+    let result = typeinfo
+        .get_harness()
+        .read_polymorphic_data(context, &typeinfo);
     context.dec_depth();
     result
 }
@@ -371,8 +373,9 @@ pub fn read_rc_any(
             };
             // Check for generic container types which cannot be deserialized polymorphically
             check_generic_container_type(&typeinfo)?;
-            let read_data_fn = typeinfo.get_harness().get_read_data_fn();
-            let boxed = read_data_fn(context)?;
+            let boxed = typeinfo
+                .get_harness()
+                .read_polymorphic_data(context, &typeinfo)?;
             context.dec_depth();
             Ok(Rc::<dyn Any>::from(boxed))
         }
@@ -385,8 +388,9 @@ pub fn read_rc_any(
             };
             // Check for generic container types which cannot be deserialized polymorphically
             check_generic_container_type(&typeinfo)?;
-            let read_data_fn = typeinfo.get_harness().get_read_data_fn();
-            let boxed = read_data_fn(context)?;
+            let boxed = typeinfo
+                .get_harness()
+                .read_polymorphic_data(context, &typeinfo)?;
             context.dec_depth();
             let rc: Rc<dyn Any> = Rc::from(boxed);
             context.ref_reader.store_rc_ref(rc.clone());
@@ -545,8 +549,9 @@ pub fn read_arc_any(
             };
             // Check for generic container types which cannot be deserialized polymorphically
             check_generic_container_type(&typeinfo)?;
-            let read_data_fn = typeinfo.get_harness().get_read_data_fn();
-            let boxed = read_data_fn(context)?;
+            let boxed = typeinfo
+                .get_harness()
+                .read_polymorphic_data(context, &typeinfo)?;
             context.dec_depth();
             Ok(Arc::<dyn Any>::from(boxed))
         }
@@ -560,8 +565,9 @@ pub fn read_arc_any(
             };
             // Check for generic container types which cannot be deserialized polymorphically
             check_generic_container_type(&typeinfo)?;
-            let read_data_fn = typeinfo.get_harness().get_read_data_fn();
-            let boxed = read_data_fn(context)?;
+            let boxed = typeinfo
+                .get_harness()
+                .read_polymorphic_data(context, &typeinfo)?;
             context.dec_depth();
             let arc: Arc<dyn Any> = Arc::from(boxed);
             context.ref_reader.store_arc_ref(arc.clone());
